@@ -18,12 +18,14 @@ class TestEndpoint(object):
   def connection(self, requestor):
     return ConnectionImpl(requestor)
 
-  def assert_called(self, requestor, connection, method, url, params=None):
+  def assert_called(self, requestor, connection, method, url, params=None, headers=None, user_agent=None):
     params = params or {}
     if method in ('get', 'delete'):
       kwargs = {'params': params, 'json': None}
     else:
       kwargs = {'json': params, 'params': None}
+    kwargs.update({'headers': headers})
+    kwargs.update({'user_agent': user_agent})
     requestor.request.assert_called_once_with(
       method.upper(),
       'https://api.sigopt.com/v1' + url,
@@ -54,6 +56,10 @@ class TestEndpoint(object):
     connection.experiments(1).best_assignments().fetch()
     self.assert_called(requestor, connection, 'get', '/experiments/1/best_assignments')
 
+  def test_experiment_best_training_runs(self, requestor, connection):
+    connection.experiments(1).best_training_runs().fetch()
+    self.assert_called(requestor, connection, 'get', '/experiments/1/best_training_runs')
+
   def test_experiment_importances(self, requestor, connection):
     connection.experiments(1).importances().fetch()
     self.assert_called(requestor, connection, 'get', '/experiments/1/importances')
@@ -116,7 +122,13 @@ class TestEndpoint(object):
 
   def test_queued_suggestion_list_params(self, requestor, connection):
     connection.experiments(1).queued_suggestions().fetch(limit=10, before='1')
-    self.assert_called(requestor, connection, 'get', '/experiments/1/queued_suggestions', {'limit': '10', 'before': '1'})
+    self.assert_called(
+      requestor,
+      connection,
+      'get',
+      '/experiments/1/queued_suggestions',
+      {'limit': '10', 'before': '1'},
+    )
 
   def test_queued_suggestion_detail(self, requestor, connection):
     connection.experiments(1).queued_suggestions(2).fetch()
@@ -170,10 +182,6 @@ class TestEndpoint(object):
     connection.experiments(1).observations().delete()
     self.assert_called(requestor, connection, 'delete', '/experiments/1/observations')
 
-  def test_plan(self, requestor, connection):
-    connection.clients(1).plan().fetch()
-    self.assert_called(requestor, connection, 'get', '/clients/1/plan')
-
   def test_token_create(self, requestor, connection):
     connection.experiments(1).tokens().create()
     self.assert_called(requestor, connection, 'post', '/experiments/1/tokens')
