@@ -1,12 +1,10 @@
 import os
-import math
 import numpy
 import pytest
 import warnings
 
 from sigopt.objects import *
-
-warnings.simplefilter("always")
+from .utils import ObserveWarnings
 
 def load(filename):
   with open(os.path.join(os.path.dirname(__file__), 'json', filename), 'r') as f:
@@ -20,6 +18,10 @@ def load_and_parse(Cls, filename):
   return obj
 
 class TestBase(object):
+  @pytest.fixture(autouse=True)
+  def set_warnings(self):
+    warnings.simplefilter("error")
+
   def test_as_json(self):
     assert ApiObject.as_json(None) is None
     assert ApiObject.as_json(False) is False
@@ -76,6 +78,9 @@ class TestBase(object):
         repr(Bounds({'max': 0.1, 'min': -0.2})) == 'Bounds(\n  max=0.1,\n  min=-0.2,\n)'
         or repr(Bounds({'max': 0.1, 'min': -0.2})) == 'Bounds(\n  min=-0.2,\n  max=0.1,\n)'
     )
+    assert repr(Pagination(Experiment, {'data': [{}]})) == (
+      'Pagination<Experiment>(\n  data=[\n    Experiment(),\n  ],\n)'
+    )
 
   def test_json(self):
     assert Experiment({}).to_json() == {}
@@ -96,7 +101,7 @@ class TestBase(object):
       a.method_that_doesnt_exist_on_dict()
 
     with pytest.raises(KeyError):
-      a['xyz']
+      a['xyz']  # pylint: disable=pointless-statement
     assert a.get('xyz') is None
     assert a.get('xyz', 'fake') == 'fake'
 
@@ -178,24 +183,6 @@ class TestObjects(object):
     assert experiment.progress.last_observation.created == 452
     assert experiment.progress.last_observation.suggestion == '12'
     assert experiment.progress.last_observation.experiment == '123'
-    assert isinstance(experiment.progress.best_observation, Observation)
-    assert experiment.progress.best_observation.id == '3'
-    assert isinstance(experiment.progress.best_observation.assignments, Assignments)
-    assert experiment.progress.best_observation.assignments.get('a') == 3
-    assert experiment.progress.best_observation.assignments.get('b') == 'd'
-    assert experiment.progress.best_observation.values[0].name == 'Revenue'
-    assert experiment.progress.best_observation.values[0].value is None
-    assert experiment.progress.best_observation.values[0].value_stddev is None
-    assert experiment.progress.best_observation.values[1].name == 'Sales'
-    assert experiment.progress.best_observation.values[1].value is None
-    assert experiment.progress.best_observation.values[1].value_stddev is None
-    assert experiment.progress.best_observation.failed is True
-    assert experiment.progress.best_observation.created == 453
-    assert experiment.progress.best_observation.suggestion == '13'
-    assert experiment.progress.best_observation.experiment == '123'
-    assert isinstance(experiment.progress.best_observation.metadata, Metadata)
-    assert experiment.progress.best_observation.metadata['abc'] == 'def'
-    assert experiment.progress.best_observation.metadata['ghi'] == 123
     assert len(experiment.parameters) == 2
     assert isinstance(experiment.parameters[0], Parameter)
     assert experiment.parameters[0].name == 'a'
@@ -229,18 +216,36 @@ class TestObjects(object):
     assert experiment.updated == 453
     assert experiment.user == '789'
 
-    with warnings.catch_warnings(record=True) as w:
+    with ObserveWarnings() as w:
       assert experiment.can_be_deleted is None
       assert len(w) == 1
       assert issubclass(w[-1].category, DeprecationWarning)
 
-    with warnings.catch_warnings(record=True) as w:
+    with ObserveWarnings() as w:
       assert experiment.parameters[0].tunable is None
       assert len(w) == 1
       assert issubclass(w[-1].category, DeprecationWarning)
 
-    with warnings.catch_warnings(record=True) as w:
-      experiment.progress.best_observation
+    with ObserveWarnings() as w:
+      best_observation = experiment.progress.best_observation
+      assert isinstance(best_observation, Observation)
+      assert best_observation.id == '3'
+      assert isinstance(best_observation.assignments, Assignments)
+      assert best_observation.assignments.get('a') == 3
+      assert best_observation.assignments.get('b') == 'd'
+      assert best_observation.values[0].name == 'Revenue'
+      assert best_observation.values[0].value is None
+      assert best_observation.values[0].value_stddev is None
+      assert best_observation.values[1].name == 'Sales'
+      assert best_observation.values[1].value is None
+      assert best_observation.values[1].value_stddev is None
+      assert best_observation.failed is True
+      assert best_observation.created == 453
+      assert best_observation.suggestion == '13'
+      assert best_observation.experiment == '123'
+      assert isinstance(best_observation.metadata, Metadata)
+      assert best_observation.metadata['abc'] == 'def'
+      assert best_observation.metadata['ghi'] == 123
       assert len(w) == 1
       assert issubclass(w[0].category, DeprecationWarning)
       assert 'best_assignments' in str(w[0].message)
@@ -281,6 +286,7 @@ class TestObjects(object):
     assert client.id == '1'
     assert client.name == 'Client'
     assert client.created == 123
+    assert client.organization == '2'
 
   def test_suggestion(self):
     suggestion = load_and_parse(Suggestion, 'suggestion.json')
@@ -316,26 +322,15 @@ class TestObjects(object):
     pagination = Pagination(Experiment, load('pagination.json'))
     assert isinstance(pagination, Pagination)
     assert pagination.count == 2
-    assert len(pagination.data) == 1
-    assert isinstance(pagination.data[0], Experiment)
     assert isinstance(pagination.paging, Paging)
     assert pagination.paging.before == '1'
     assert pagination.paging.after == '2'
-
-  def test_plan(self):
-    plan = load_and_parse(Plan, 'plan.json')
-    assert isinstance(plan, Plan)
-    assert plan.id == 'premium'
-    assert plan.name == 'SigOpt Premium'
-    assert isinstance(plan.rules, PlanRules)
-    assert plan.rules.max_dimension == 1
-    assert plan.rules.max_experiments == 2
-    assert plan.rules.max_observations == 3
-    assert plan.rules.max_parallelism == 4
-    assert isinstance(plan.current_period, PlanPeriod)
-    assert plan.current_period.start == 0
-    assert plan.current_period.end == 1000
-    assert plan.current_period.experiments == ['1', '2']
+    with ObserveWarnings() as w:
+      data = pagination.data
+      assert len(data) == 1
+      assert isinstance(data[0], Experiment)
+      assert len(w) == 1
+      assert issubclass(w[-1].category, RuntimeWarning)
 
   def test_token(self):
     token = load_and_parse(Token, 'token.json')
@@ -345,11 +340,15 @@ class TestObjects(object):
     assert token.development is True
     assert token.experiment == '1'
     assert token.expires == 1547139000
-    assert token.permissions == 'read'
     assert token.token == '123'
     assert token.token_type == 'client-dev'
     assert token.user == '789'
 
+    with ObserveWarnings() as w:
+      assert token.permissions == 'read'
+      assert len(w) == 1
+      assert issubclass(w[-1].category, DeprecationWarning)
+
   def test_metric(self):
     metric = load_and_parse(Metric, 'metric.json')
     assert isinstance(metric, Metric)
@@ -384,3 +383,39 @@ class TestObjects(object):
     assert isinstance(task, Task)
     assert task.name == 'task 1'
     assert task.cost == 0.567
+
+  def test_training_run(self):
+    run = load_and_parse(TrainingRun, 'training_run.json')
+    assert run.assignments['m'] == 1
+    assert run.assignments['n'] == 2
+    assert run.best_checkpoint == 'cp0'
+    assert run.client == '11674'
+    assert run.checkpoint_count == 2
+    assert run.completed == 1631654369
+    assert run.created == 1631654365
+    assert run.datasets == ['dataset1', 'dataset2']
+    assert run.deleted is False
+    assert run.experiment == '432979'
+    assert run.files == ['f0', 'f1']
+    assert run.finished is True
+    assert run.id == '95658'
+    assert run.logs['stderr'] == 'message stderr\n'
+    assert run.logs['stdout'] == 'message stdout\n'
+    assert run.metadata['m0'] == 'v0'
+    assert run.metadata['m1'] == 'v1'
+    assert run.model.type == 'type0'
+    assert run.name == 'run-examples 2021-09-14 14:19:26'
+    assert run.object == 'training_run'
+    assert run.observation == '31818393'
+    assert run.project == 'run-examples'
+    assert run.source_code.content == 'c0'
+    assert run.source_code.hash == 'h0'
+    assert run.state == 'completed'
+    assert run.suggestion == '46227605'
+    assert run.tags == ['t0', 't1']
+    assert run.updated == 1631654369
+    assert run.user == '53628'
+    assert run.values['accuracy'].value == 1
+    assert run.values['accuracy'].value_stddev == 0.1
+    assert run.values['f1'].value == 2
+    assert run.values['f1'].value_stddev == 0.2
